#include "Plotter/PlotAceData.hpp"

using namespace frendy;

//constructor
PlotAceData::PlotAceData(void)
{
  clear();
}

//destructor
PlotAceData::~PlotAceData(void)
{
  clear();
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void PlotAceData::clear()
{
  temp_ori = -1.0;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void PlotAceData::set_xs_type_list_fast(FastAceDataObject& ace_obj,  vector<Integer>& xs_type_list)
{
  if( static_cast<int>(xs_type_list.size()) != 0 )
  {
    return;
  }

  xs_type_list.push_back(  1); //total
  xs_type_list.push_back(  2); //scatter;
  xs_type_list.push_back(101); //disappearance

  int fis_flg = 0;
  vector<int> xs_type_list_add = ace_obj.get_react_type_list();
  for(int i=0; i<static_cast<int>(xs_type_list_add.size()); i++)
  {
    xs_type_list.push_back(xs_type_list_add[i]);
    if( xs_type_list_add[i] == 18 )
    {
      fis_flg = 1;
    }
  }
  xs_type_list_add.clear();


  if( fis_flg > 0 )
  {
    //For chi (fission spectrum)
    xs_type_list.push_back(1018);

    //For nu_tot (total number of neutrons per fission)
    if( static_cast<int>(ace_obj.get_nu_data().size()) > 0 )
    {
      int mt_add = static_cast<int>(ace_obj.get_nu_bar_repr_flg())*1000 + 452;
      xs_type_list.push_back(mt_add);
    }

    //For nu_f
    if( static_cast<int>(ace_obj.get_nu_d_data().size()) > 0 &&
        static_cast<int>(ace_obj.get_nu_d_bar_repr_flg()) == 2 )
    {
      int mt_add = static_cast<int>(ace_obj.get_nu_d_bar_repr_flg())*1000 + 455;
      xs_type_list.push_back(mt_add);
    }

    //For nu_p
    if( static_cast<int>(ace_obj.get_nu_p_data().size()) > 0 )
    {
      int mt_add = static_cast<int>(ace_obj.get_nu_p_bar_repr_flg())*1000 + 456;
      xs_type_list.push_back(mt_add);
    }
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void PlotAceData::read_nu_data(FastAceDataObject& ace_obj, int mt_no, vector<Real>& e_vec, vector<Real>& nu_vec)
{
  ace_obj.get_xs_data(1, e_vec, nu_vec);
  nu_vec.clear();

  vector<Real> nu_coef;
  if( mt_no == 1452 )
  {
    nu_coef = ace_obj.get_nu_data();
  }
  else if( mt_no == 1456 )
  {
    nu_coef = ace_obj.get_nu_p_data();
  }
  else if( mt_no == 1455 )
  {
    nu_coef = ace_obj.get_nu_d_data();
  }
  else
  {
    return;
  }

  int i_max = static_cast<int>(e_vec.size());
  int j_max = static_cast<int>(nu_coef.size());
  if( j_max == 0 )
  {
    e_vec.clear();
    nu_vec.clear();
    return;
  }

  nu_vec.resize(i_max);
  for(int i=0; i<i_max; i++)
  {
    Real e_coef = 1.0;
    nu_vec[i]   = 0.0;
    for(int j=0; j<j_max; j++)
    {
      nu_vec[i] += nu_coef[j] * e_coef;
      e_coef    *= e_vec[i];
    }
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void PlotAceData::read_ace_file_fast(string file_name, vector<int>& xs_type_list,
                                     vector<vector<Real> >& e_int_xs, vector<vector<Real> >& xs_tab)
{
  //Read ACE file
  FastAceDataParser read_obj;
  read_obj.set_file_name(file_name);
  FastAceDataObject ace_obj = read_obj.get_ace_data_obj();

  set_xs_data_fast(ace_obj, xs_type_list, e_int_xs, xs_tab);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void PlotAceData::set_xs_data_fast(frendy::FastAceDataObject& ace_obj, vector<int>& xs_type_list,
                                   vector<vector<Real> >& e_int_xs, vector<vector<Real> >& xs_tab)
{
  set_xs_type_list_fast(ace_obj, xs_type_list);

  int i_max = static_cast<int>(xs_type_list.size());

  //Resize xs data
  e_int_xs.resize(i_max);
  xs_tab.resize(i_max);

  for(int i=0; i<i_max; i++)
  {
    if( xs_type_list[i] < 1000 )
    {
      ace_obj.get_xs_data(xs_type_list[i], e_int_xs[i], xs_tab[i]);
    }
    else
    {
      if( xs_type_list[i] == 1018 )
      {
        ace_obj.get_chi_data(e_int_xs[i], xs_tab[i]);
      }
      else if( xs_type_list[i] < 2000 )
      {
        read_nu_data(ace_obj, xs_type_list[i], e_int_xs[i], xs_tab[i]);
      }
      else if( xs_type_list[i] == 2452 )
      {
        e_int_xs[i] = ace_obj.get_nu_ene_data();
        xs_tab[i]   = ace_obj.get_nu_data();
      }
      else if( xs_type_list[i] == 2455 )
      {
        e_int_xs[i] = ace_obj.get_nu_d_ene_data();
        xs_tab[i]   = ace_obj.get_nu_d_data();
      }
      else if( xs_type_list[i] == 2456 )
      {
        e_int_xs[i] = ace_obj.get_nu_p_ene_data();
        xs_tab[i]   = ace_obj.get_nu_p_data();
      }
    }

    int j_max = static_cast<int>(e_int_xs[i].size());
    for(int j=0; j<j_max; j++)
    {
      e_int_xs[i][j] *= 1.0E6; //MeV -> eV
    }
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void PlotAceData::set_xs_type_list_tsl(ThermalScatterAceDataObject& ace_obj, vector<Integer>& xs_type_list)
{
  if( static_cast<int>(xs_type_list.size()) != 0 )
  {
    return;
  }

  if( static_cast<int>(ace_obj.get_inelastic_xs_data().size()) > 0 )
  {
    xs_type_list.push_back(221); //inelastic scattering
  }

  if( static_cast<int>(ace_obj.get_coh_elastic_e_xs_data().size()) > 0 )
  {
    xs_type_list.push_back(222); //coherent elastic scattering
  }

  if( static_cast<int>(ace_obj.get_incoh_elastic_xs_data().size()) > 0 )
  {
    xs_type_list.push_back(223); //incoherent elastic scattering
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void PlotAceData::read_ace_file_tsl(string file_name, vector<int>& xs_type_list,
                                    vector<vector<Real> >& e_int_xs, vector<vector<Real> >& xs_tab)
{
  //Read ACE file
  ThermalScatterAceDataParser read_obj;
  read_obj.set_file_name(file_name);
  ThermalScatterAceDataObject ace_obj = read_obj.get_ace_data_obj();

  set_xs_data_tsl(ace_obj, xs_type_list, e_int_xs, xs_tab);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void PlotAceData::set_xs_data_tsl(frendy::ThermalScatterAceDataObject& ace_obj,
                                 vector<Integer>& xs_type_list,
                                 vector<vector<Real> >& e_int_xs, vector<vector<Real> >& xs_tab)
{
  set_xs_type_list_tsl(ace_obj, xs_type_list);

  int i_max = static_cast<int>(xs_type_list.size());
  vector<Real> coh_ene   = ace_obj.get_coh_elastic_ene_data();
  vector<Real> coh_xs    = ace_obj.get_coh_elastic_e_xs_data();
  vector<Real> incoh_ene = ace_obj.get_incoh_elastic_ene_data();
  vector<Real> incoh_xs  = ace_obj.get_incoh_elastic_xs_data();
  vector<Real> inela_ene = ace_obj.get_inelastic_ene_data();
  vector<Real> inela_xs  = ace_obj.get_inelastic_xs_data();

  //Resize xs data
  e_int_xs.resize(i_max);
  xs_tab.resize(i_max);

  for(int i=0; i<i_max; i++)
  {
    if( xs_type_list[i] == 221 )
    {
      e_int_xs[i] = inela_ene;
      xs_tab[i]   = inela_xs;
    }

    if( xs_type_list[i] == 222 )
    {
      e_int_xs[i] = coh_ene;
      xs_tab[i]   = coh_xs;
    }

    if( xs_type_list[i] == 223 )
    {
      e_int_xs[i] = incoh_ene;
      xs_tab[i]   = incoh_xs;
    }

    int j_max = static_cast<int>(e_int_xs[i].size());
    for(int j=0; j<j_max; j++)
    {
      e_int_xs[i][j] *= 1.0E6; //MeV -> eV
    }
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void PlotAceData::plot_ace(string file_name, vector<Integer>& xs_type_list,
                           vector<vector<Real> >& e_int_xs, vector<vector<Real> >& xs_tab)
{
  AceFileReader read_obj;
  int ace_type_no = read_obj.get_ace_data_type_from_ace_file(file_name);
      temp_ori    = read_obj.get_temp(file_name);

  if( ace_type_no == read_obj.fast_data )
  {
    read_ace_file_fast(file_name, xs_type_list, e_int_xs, xs_tab);
  }
  else if( ace_type_no == read_obj.tsl_data )
  {
    read_ace_file_tsl(file_name, xs_type_list, e_int_xs, xs_tab);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void PlotAceData::plot_ace(string file_name, vector<Integer>& xs_type_list,
                           vector<vector<Real> >& e_int_xs, vector<vector<Real> >& xs_tab,
                           Real temp)
{
  AceFileReader read_obj;
  int ace_type_no = read_obj.get_ace_data_type_from_ace_file(file_name);
      temp_ori    = read_obj.get_temp(file_name);

  if( ace_type_no == read_obj.fast_data )
  {
    read_ace_file_fast(file_name, xs_type_list, e_int_xs, xs_tab);
  }
  else if( ace_type_no == read_obj.tsl_data )
  {
    read_ace_file_tsl(file_name, xs_type_list, e_int_xs, xs_tab);
  }

  if( temp > -1.0 * min_value )
  {
    //Doppler broadening
    Real awr_ace  = read_obj.get_awr(file_name);
    cout << "Doppler broadening from " << temp_ori << " [K] to " << temp << " [K]." << endl;
    calc_dop_obj.calc_doppler_broadened_xs_from_xs_table
      (temp_ori, temp, awr_ace, xs_type_list,  e_int_xs, xs_tab);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

Real PlotAceData::get_temp_ori()
{
  return temp_ori;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

