#include "Plotter/PlotEndfData.hpp"

using namespace frendy;

//constructor
PlotEndfData::PlotEndfData(void)
{
  clear();
}

//destructor
PlotEndfData::~PlotEndfData(void)
{
  clear();
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void PlotEndfData::clear()
{
  temp_ori = -1.0;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void PlotEndfData::set_xs_type_list(Endf6Parser& endf_obj,  vector<Integer>& xs_type_list)
{
  if( static_cast<int>(xs_type_list.size()) != 0 )
  {
    return;
  }

  //Get MF3 data
  vector<MF03Parser> mf03 = endf_obj.get_mf03_data();

  xs_type_list.clear();
  for(int i=0; i<static_cast<int>(mf03.size()); i++)
  {
    Integer mt_no = mf03[i].get_mt_no();
    if( mt_no != unassigned_mt_no )
    {
      xs_type_list.push_back(mt_no);
    }
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void PlotEndfData::set_xs_data(Endf6Parser& endf_obj,  vector<Integer> xs_type_list,
                              vector<vector<Real> >& e_int_xs, vector<vector<Real> >& xs_tab)
{
  //Resize xs data
  int xs_data_no = static_cast<int>(xs_type_list.size());
  e_int_xs.resize(xs_data_no);
  xs_tab.resize(xs_data_no);

  //Linearizing TAB1 data
  LinearizeTabData  lin_tab_obj;

  //Get MF3 data
  vector<MF03Parser> mf03 = endf_obj.get_mf03_data();
  for(int i=0; i<static_cast<int>(mf03.size()); i++)
  {
    Integer mt_no = mf03[i].get_mt_no();
    for(int j=0; j<xs_data_no; j++)
    {
      if( mt_no == xs_type_list[j] )
      {
        vector<Integer> range_vec = mf03[i].get_NBT_SIG();
        vector<Integer> int_vec   = mf03[i].get_INT_SIG();
        vector<Real>    ene_vec   = mf03[i].get_E_INT_SIG();
        vector<Real>    xs_vec    = mf03[i].get_SIG_TAB();
        lin_tab_obj.linearize_tab1_data(range_vec, int_vec, ene_vec, xs_vec);

        e_int_xs[j] = ene_vec;
        xs_tab[j]   = xs_vec;
        break;
      }
    }
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void PlotEndfData::resonance_reconstruction(Endf6Parser& endf_obj, Real err, Real err_max, Real err_int)
{
  //Convert Endf6Parser object to NuclearDataObject using Endf6Converter
  NuclearDataObject nucl_data_obj;
  Endf6Converter    conv_obj;
  conv_obj.convert_endf_format_to_frendy(endf_obj, nucl_data_obj);

  //*** Resonance reconstruction ***
  //Check interpolation scheme of all cross section data
  int linearization_flg = 0;
  vector<CrossSectionDataContainer> xs_data_obj;
  xs_data_obj = nucl_data_obj.get_nucl_reaction_data_obj().get_xs_data_obj_assigned();
  int i_max = static_cast<int>(xs_data_obj.size());
  for(int i=0; i<i_max; i++)
  {
    vector<Integer> int_vec = xs_data_obj[i].get_xs_int_data();

    int j_max = static_cast<int>(int_vec.size());
    for(int j=0; j<j_max; j++)
    {
      if( int_vec[j] != int_lin_lin )
      {
        linearization_flg++;
        break;
      }
    }

    if( linearization_flg > 0 ) break;
  }
  vector<CrossSectionDataContainer>().swap(xs_data_obj);

  int reso_flg = 0;
  vector<vector<Integer> > lru = endf_obj.get_mf02_mt151_data().get_LRU();
  for(int i=0; i<static_cast<int>(lru.size()); i++)
  {
    for(int j=0; j<static_cast<int>(lru[i].size()); j++)
    {
      if( lru[i][j] > 0 )
      {
        reso_flg++;
      }
    }
  }
  lru.clear();

  int mod_flg = 0;
  if( reso_flg > 0 || linearization_flg > 0 )
  {
    cout << "  Resonance reconstruction" << endl;
    mod_flg++;

    ResonanceXSReconstructor  recon_obj;
    recon_obj.clear();

    //Set NuclearDataObject and input data to ResonanceXSReconstructor 
    recon_obj.set_nucl_data_obj(nucl_data_obj);
    recon_obj.set_modify_photon_data_flg(1); //modify photon data
    recon_obj.set_err(err);
    recon_obj.set_err_int(err_int);
    recon_obj.set_err_max(err_max);

    //Run ResonanceXSReconstructor
    recon_obj.output_set_data_information();
    recon_obj.calc_reconstruct_resonance_xs();

    //Get calculation result
    nucl_data_obj = recon_obj.get_nucl_data_obj();
    recon_obj.clear();
    cout << endl;
  }

  if( mod_flg > 0 )
  {
    //Convert NuclearDataObject to Endf6Parser object using Endf6Converter
    endf_obj.clear();
    conv_obj.convert_frendy_to_endf_format(nucl_data_obj, endf_obj);
    nucl_data_obj.clear();
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void PlotEndfData::plot_endf(Endf6Parser& endf_obj, vector<Integer>& xs_type_list,
                             vector<vector<Real> >& e_int_xs, vector<vector<Real> >& xs_tab,
                             Real err, Real err_max, Real err_int)
{
  resonance_reconstruction(endf_obj, err, err_max, err_int);

  if( static_cast<int>(xs_type_list.size()) == 0 )
  {
    set_xs_type_list(endf_obj, xs_type_list);
  }

  set_xs_data(endf_obj, xs_type_list, e_int_xs, xs_tab);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void PlotEndfData::plot_endf(string file_name, vector<Integer>& xs_type_list,
                             vector<vector<Real> >& e_int_xs, vector<vector<Real> >& xs_tab,
                             Integer mat, Real temp, Real err, Real err_max, Real err_int)
{
  if( mat <= 0 )
  {
    plot_endf(file_name, xs_type_list, e_int_xs, xs_tab, temp, err, err_max, err_int);
    return;
  }


  EndfTapeDivider div_obj;
  div_obj.set_file_name(file_name);
  vector<string> text_vec = div_obj.get_nearby_temp_data(mat, temp);
  if( static_cast<int>(text_vec.size()) == 0 )
  {
    return;
  }


  //Allocate Endf6Parser class to endf_obj;
  Endf6Parser endf_obj;

  //Set pendf file name
  endf_obj.set_text_no_cov(text_vec);
  temp_ori = endf_obj.get_mf01_mt451_data().get_TEMP();

  plot_endf(endf_obj, xs_type_list, e_int_xs, xs_tab, temp, err, err_max, err_int);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void PlotEndfData::plot_endf(string file_name, vector<Integer>& xs_type_list,
                             vector<vector<Real> >& e_int_xs, vector<vector<Real> >& xs_tab,
                             Real temp, Real err, Real err_max, Real err_int)
{
  //Allocate Endf6Parser class to endf_obj;
  Endf6Parser endf_obj;

  //Set pendf file name
  endf_obj.set_file_name_no_cov(file_name);
  temp_ori = endf_obj.get_mf01_mt451_data().get_TEMP();

  plot_endf(endf_obj, xs_type_list, e_int_xs, xs_tab, temp, err, err_max, err_int);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void PlotEndfData::plot_endf(Endf6Parser& endf_obj, vector<Integer>& xs_type_list,
                             vector<vector<Real> >& e_int_xs, vector<vector<Real> >& xs_tab,
                             Real temp, Real err, Real err_max, Real err_int)
{
  plot_endf(endf_obj, xs_type_list, e_int_xs, xs_tab, err, err_max, err_int);

  if( temp > -1.0 * min_value )
  {
    //Doppler broadening
    Real awr      = endf_obj.get_mf01_mt451_data().get_AWR();
    cout << "Doppler broadening from " << temp_ori << " [K] to " << temp << " [K]." << endl;
    calc_dop_obj.calc_doppler_broadened_xs_from_xs_table
      (temp_ori, temp, awr, xs_type_list, e_int_xs, xs_tab);
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void PlotEndfData::plot_endf(string file_name, vector<Integer>& xs_type_list,
                             vector<vector<Real> >& e_int_xs, vector<vector<Real> >& xs_tab,
                             Real err, Real err_max, Real err_int)
{
  //Allocate Endf6Parser class to endf_obj;
  Endf6Parser endf_obj;

  //Set pendf file name
  endf_obj.set_file_name_no_cov(file_name);

  plot_endf(endf_obj, xs_type_list, e_int_xs, xs_tab, err, err_max, err_int);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void PlotEndfData::plot_endf(Endf6Parser& endf_obj, vector<Integer>& xs_type_list,
                             vector<vector<Real> >& e_int_xs, vector<vector<Real> >& xs_tab)
{
  Real err     = 1.0E-3;
  Real err_max = err * 10.0;
  Real err_int = err / 20000.0;

  plot_endf(endf_obj, xs_type_list, e_int_xs, xs_tab, err, err_max, err_int);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void PlotEndfData::plot_endf(string file_name, vector<Integer>& xs_type_list,
                             vector<vector<Real> >& e_int_xs, vector<vector<Real> >& xs_tab)
{
  //Allocate Endf6Parser class to endf_obj;
  Endf6Parser endf_obj;

  //Set pendf file name
  endf_obj.set_file_name_no_cov(file_name);

  plot_endf(endf_obj, xs_type_list, e_int_xs, xs_tab);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

Real PlotEndfData::get_temp_ori()
{
  return temp_ori;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

