#include "ReconResonance/RmatrixLimited/numeric/Parameter.h"
#include "ReconResonance/RmatrixLimited/numeric/Approx.h"

//ClassImp( Numeric::Parameter );

namespace Numeric {


/*! @brief Default constructor */
Parameter::Parameter() : value(0.), error(0.), text("") {}

/*! @brief Constructor */
Parameter::Parameter( const double val, const double err, const string name ) {

  this->Set( val, err, name );

}

/*! @brief Constructor */
Parameter::Parameter( const Parameter& o ) {

  this->Set( o );

}

/*! @brief Destructor */
Parameter::~Parameter(){}



Parameter* Parameter::Set( const double val, const double err, const string name ) {

  value = val;  error = err;  text = name;  return this;

}


Parameter* Parameter::Set( const Parameter& o ) {

  this->Set( o.value, o.error, o.text );  return this;

}


/*! @brief Clear this object */
void Parameter::Clear() {

  value = 0.;  error = 0.;  text.clear();


}


Parameter& Parameter::operator*=( const double val ) {

  value *= val;  error *= val;  return *this;

}

Parameter& Parameter::operator/=( const double val ) {

  value /= val;  error /= val;  return *this;

}

Parameter& Parameter::operator*=( const Parameter& o ) {

  error  = sqrt( pow( o.Value()*error, 2. ) + pow( value*o.Error(), 2. ) );
  value *= o.Value();

  return *this;

}

Parameter& Parameter::operator/=( const Parameter& o ) {

  Parameter temp;

    temp.Value() = 1./o.Value();
    temp.Error() = o.Error()/o.Value()/o.Value();

  return this->operator*=( temp );

}

bool Parameter::operator==( const Parameter& o ) const {

  return ( eq( this->Value(), o.Value() )
        && eq( this->Error(), o.Error() )
        && this->Name() == o.Name() );

}

bool Parameter::operator!=( const Parameter& o ) const {

  return !( *this == o );

}


double Parameter::Perturb( const double eps ) {

  double delta = fabs(value)*eps;  value += delta;

  return delta;

}




void Parameter::Print( ostream& os ) const {

  os.setf(ios::fixed, ios::floatfield);

  os << setfill(' ') << internal << right << setw(8) << setprecision(5) << this->Value();

  if( error == 0.0 ) os << "           ";
  else os << " ± " << fixed << setfill(' ') << setw(6) << setprecision(3) << this->ErrorPercent() << " %";

  return;

}


void Parameter::PrintLaTex( ostream& os ) const {

  os.setf(ios::fixed, ios::floatfield);

  os << '$' << setfill(' ') << internal << right << setw(8) << setprecision(5) << this->Value() << "$ ";


  if( error == 0.0 ) os << "          ";
  else os << "$\\pm " << setw(6) << setprecision(3) << this->ErrorPercent() << " \\%$";

  return;

}



ostream& operator<<( ostream& os, const Parameter& o ) {

  o.Print( os );  return os;

}


}
